library IEEE;
use IEEE.STD_LOGIC_1164.ALL;
use IEEE.NUMERIC_STD.ALL;

entity RAM_32x40_tb is
    --  Port ( );
end RAM_32x40_tb;

architecture Behavioral of RAM_32x40_tb is

    signal clk        : std_logic := '0';
    signal we         : std_logic := '0';
    signal addr_write : std_logic_vector (4 downto 0);
    signal addr_read  : std_logic_vector (4 downto 0);
    signal data_write : std_logic_vector (0 to 39);
    signal data_read  : std_logic_vector (0 to 39);

    constant CLK_PERIOD : time := 10 ns;

begin
    UUT: entity work.RAM_32x40(Behavioral)
        port map (
            clk        => clk,
            we         => we,
            addr_write => addr_write,
            addr_read  => addr_read,
            data_write => data_write,
            data_read  => data_read
        );

    STIM_CLK: process
    begin
        wait for CLK_PERIOD/2;
        clk <= not clk;
    end process;

    STIM: process
    begin

        wait for CLK_PERIOD*2;

        -- initialize/write contents
        we <= '0';
        data_write <= X"F0F0F0F0F0";
        for i in 0 to 31 loop
            we <= '1';
            addr_write <= std_logic_vector(to_unsigned(i,addr_write'length));
            wait for CLK_PERIOD;
            we <= '0';
            wait for CLK_PERIOD;
        end loop;

        -- read
        we <= '0';
        addr_write <= (others => '0');
        for i in 0 to 31 loop
            addr_read <= std_logic_vector(to_unsigned(i,addr_read'length));
            wait for CLK_PERIOD/4;
        end loop;

        -- write and read        
        for i in 0 to 31 loop
            wait for CLK_PERIOD;
            we <= '1';
            addr_write <= std_logic_vector(to_unsigned(i,addr_write'length));
            data_write <= std_logic_vector(to_unsigned(i,data_write'length));
            wait for CLK_PERIOD;
            we <= '0';
            addr_read <= std_logic_vector(to_unsigned(i,addr_read'length));
        end loop;

        wait;
    end process;
end Behavioral;
